import logging
import os
import shutil

from .. import Component
from zygoat.utils.shell import run
from zygoat.constants import Projects

from .dockerfile import dockerfile
from .wait_command import wait_command
from .settings import settings
from .gitignore import gitignore
from .black import black
from .flake8 import flake8
from .banditrc import banditrc
from .pytest import pytest
from .dependencies import dependencies
from .reformat import reformat

log = logging.getLogger()


class Backend(Component):
    def create(self):
        log.info("Installing django at a user level to generate the project")
        run(["pip", "install", "--user", "--upgrade", "django"])

        log.info("Creating the django project")
        run(["django-admin", "startproject", Projects.BACKEND])

    def delete(self):
        log.warning(f"Deleting the {Projects.BACKEND} project")
        shutil.rmtree(Projects.BACKEND)

    @property
    def installed(self):
        return os.path.exists(Projects.BACKEND)


backend = Backend(
    sub_components=[
        settings,
        dockerfile,
        dependencies,
        wait_command,
        gitignore,
        black,
        banditrc,
        flake8,
        pytest,
        # It is important that this comes last, so all relevant items can be reformatted
        reformat,
    ]
)



import os

from zygoat.constants import Projects
from zygoat.components import FileComponent

from . import resources


class Proxy(FileComponent):
    resource_pkg = resources
    filename = "[...slug].js"
    base_path = os.path.join(Projects.FRONTEND, "pages", "api")
    overwrite = False


proxy = Proxy()



import logging
import os

from zygoat.components import Component, FileComponent, SettingsComponent
from zygoat.constants import Projects

from . import resources

log = logging.getLogger()

import_path = "backend.proxy.ReverseProxyHandlingMiddleware"


class ProxyFile(FileComponent):
    resource_pkg = resources
    base_path = os.path.join(Projects.BACKEND, Projects.BACKEND)
    filename = "proxy.py"
    overwrite = False


class SettingsEntry(SettingsComponent):
    def create(self):
        red = self.parse()
        middleware_list = red.find("name", "MIDDLEWARE").parent.value

        log.info("Adding reverse proxy middleware")
        middleware_list.insert(0, f"'{import_path}'")

        self.dump(red)

    @property
    def installed(self):
        red = self.parse()
        middleware_list = red.find("name", "MIDDLEWARE").parent.value.to_python()

        return import_path in middleware_list


# Just a shell for hosting the other two components
class ReverseProxy(Component):
    pass


reverse_proxy = ReverseProxy(sub_components=[ProxyFile(), SettingsEntry()])



from contextlib import contextmanager
import os
import logging

from click import style

log = logging.getLogger()


def walk_up():
    """
    Generator expression to provide paths up to the system root

    Does not work on Windows, but then again, neither does Zygoat
    """
    path = os.getcwd()

    while True:
        log.debug(f"Searching {style(path, bold=True)}")
        yield path

        if path == "/":
            raise FileNotFoundError

        path = os.path.dirname(path)


def find_nearest(file_name):
    """
    Returns the absolute path to the nearest existing file matching file_name

    :param file_name: Name of the file to locate
    """
    try:
        for path in walk_up():
            target = os.path.join(path, file_name)

            # If the file is not found, walk_up() will error out
            if os.path.exists(target):
                log.debug(f"Found {file_name} in {os.path.dirname(target)}")
                return target
    except FileNotFoundError:
        raise FileNotFoundError(
            f"Unable to locate {file_name} in current or any parent directory"
        )


@contextmanager
def use_dir(path):
    """
    A context manager for switching into an arbitrary directory for a block

    :param path: A valid directory path
    """
    owd = os.getcwd()

    try:
        os.chdir(path)
        yield
    finally:
        os.chdir(owd)


@contextmanager
def repository_root():
    """
    A shortcut for locating the nearest repository root and doing a use_dir with it
    """

    root = os.path.dirname(find_nearest(".git"))

    with use_dir(root):
        yield



class ReverseProxyHandlingMiddleware(object):
    """
    Normalize all incoming IP addresses from the load balancer
    """

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if "HTTP_X_FORWARDED_FOR" in request.META:
            ip = request.META["HTTP_X_FORWARDED_FOR"]
        else:
            ip = request.META["REMOTE_ADDR"]

        request.META["REMOTE_ADDR"] = ip.split(",")[0]

        return self.get_response(request)



from zygoat.components import FileComponent
from zygoat.constants import Projects

from . import resources


class PrettierRc(FileComponent):
    filename = ".prettierrc"
    resource_pkg = resources
    base_path = Projects.FRONTEND


prettierrc = PrettierRc()



import logging

from zygoat.components import SettingsComponent

log = logging.getLogger()

security_headers = """\
if not DEBUG:
    SECURE_BROWSER_XSS_FILTER = True
    SECURE_CONTENT_TYPE_NOSNIFF = True
    SECURE_HSTS_SECONDS = 31536000
    SECURE_HSTS_INCLUDE_SUBDOMAINS = True
"""


class Security(SettingsComponent):
    def create(self):
        red = self.parse()

        log.info("Adding security headers")

        red.extend(
            ["\n", "# Set security headers", "\n", security_headers,]
        )
        self.dump(red)

    @property
    def installed(self):
        red = self.parse()
        return red.find("name", value="SECURE_HSTS_SECONDS") is not None


security = Security()
